from .classes import *

import aiohttp

async def request(filename: str, creds: KomfoventCredentials) -> tuple[KomfoventConnectionResult, str]:
    try:
        async with aiohttp.ClientSession(timeout=aiohttp.ClientTimeout(5.0)) as session:
            async with session.post(creds.host() + filename, data= {'1': creds.username, '2': creds.password}) as resp:
                return await validate_komfovent_response(resp)
    except:
        return (KomfoventConnectionResult.NOT_FOUND, None)

async def validate_komfovent_response(resp: aiohttp.ClientResponse,) -> tuple[KomfoventConnectionResult, str]:
    text = await resp.text()
    if resp.status != 200:
        return (KomfoventConnectionResult.NOT_FOUND, None)
    if resp.headers.get('Server') != 'C6':
        return (KomfoventConnectionResult.NOT_FOUND, None)
    if "value=\"login\"" in text.replace(" ", "").lower():
        return (KomfoventConnectionResult.UNAUTHORISED, None)
    return (KomfoventConnectionResult.SUCCESS, text)

async def update(creds: KomfoventCredentials, command: KomfoventCommand, value) -> None:
    try:
        async with aiohttp.ClientSession(timeout=aiohttp.ClientTimeout(5.0)) as session:
            async with session.post(creds.host() + "/ajax.xml", data=f"{command.value}={value}"):
                return
    except:
        return


from .classes import *
from .networking import request, update

import lxml.html as LX

# helpers

def get_credentials(host: str, username: str, password: str) -> tuple[KomfoventConnectionResult, KomfoventCredentials]:
    try:
        return (KomfoventConnectionResult.SUCCESS, KomfoventCredentials(host, username, password))
    except:
        return (KomfoventConnectionResult.INVALID_INPUT, None)

def __string_to_float(input: str) -> float:
    return float(''.join(c for c in input if c.isdigit() or c == '.'))

# check_settings

async def get_settings(creds: KomfoventCredentials) -> tuple[KomfoventConnectionResult, KomfoventSettins]:
    status, response = await request("/st.html", creds)
    if (status != KomfoventConnectionResult.SUCCESS):
        return (status, None)
    return (status, __parse_settings(response))

def __parse_settings(data: str) -> KomfoventSettins:
    root = LX.fromstring(data)
    name = root.xpath("//input[@name='204']")[0].value.strip()
    model = root.find_class("hide_cfg")[0].xpath("input")[0].value.strip()
    version = root.get_element_by_id("mmf1").value.strip()
    serial_number = root.get_element_by_id("c6sn").value.strip()
    return KomfoventSettins(name, model, version, serial_number)

# get_unit_status

async def get_unit_status(creds: KomfoventCredentials) -> tuple[KomfoventConnectionResult, KomfoventUnit]:
    status, response = await request("/i.asp", creds)
    if (status != KomfoventConnectionResult.SUCCESS):
        return (status, None)
    return (status, __parse_unit_status(response))

def __parse_unit_status(data: str) -> KomfoventUnit:
    root = LX.fromstring(data.encode())
    return KomfoventUnit(
        mode = root.xpath("omo")[0].text.strip().upper(),
        fan_speed = int(root.xpath("sp")[0].text.strip()),
        temp_target = __string_to_float(root.xpath("st")[0].text),
        temp_supply = __string_to_float(root.xpath("ai0")[0].text),
        temp_extract = __string_to_float(root.xpath("ai1")[0].text),
        temp_outside = __string_to_float(root.xpath("ai2")[0].text),
    )

async def set_operating_mode(creds: KomfoventCredentials, mode: KomfoventOperatingModes) -> KomfoventConnectionResult:
    auth_status, _ = await get_settings(creds)
    if (auth_status != KomfoventConnectionResult.SUCCESS):
        return auth_status
    await update(creds, KomfoventCommand.SET_MODE, mode.value)
    return KomfoventConnectionResult.SUCCESS


from enum import Enum
from dataclasses import dataclass
from urllib.parse import urlparse, ParseResult


class KomfoventConnectionResult(Enum):
    SUCCESS = 1
    NOT_FOUND = 2
    UNAUTHORISED = 3
    INVALID_INPUT = 4

class KomfoventCredentials():
  
    __host: ParseResult = None
    username = None
    password = None

    def __init__(self, host: str, username: str, password: str):
        if "://" in host:
            self.__host = urlparse(host)
        else:
            self.__host = urlparse("//" + host)
        self.username = username
        self.password = password

    def host(self) -> str:
        return "http://" + self.__host.netloc

class KomfoventCommand(Enum):
    SET_MODE = 3

class KomfoventOperatingModes(Enum):
    AWAY = 1
    NORMAL = 2
    INTENSIVE = 3
    BOOST = 4


@dataclass
class KomfoventSettins():
    name: str
    model: str
    version: str
    serial_number: str


@dataclass
class KomfoventUnit():
    mode: str
    fan_speed: int
    temp_target: float
    temp_supply: float
    temp_extract: float
    temp_outside: float


"""Implementing Komfovent C6 API in python"""

from .api import get_credentials, get_settings, get_unit_status, set_operating_mode
from .classes import KomfoventConnectionResult, KomfoventSettins, KomfoventOperatingModes, KomfoventCredentials